@@ -19,13 +19,13 @@ import time
 from heatclient.common import deployment_utils
 from heatclient import exc as heat_exc
 from mistral.workflow import utils as mistral_workflow_utils
-from mistralclient.api import base as mistralclient_exc
 from swiftclient import exceptions as swiftexceptions
 
 from tripleo_common.actions import base
 from tripleo_common.actions import templates
 from tripleo_common import constants
 from tripleo_common.utils import overcloudrc
+from tripleo_common.utils import plan as plan_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -145,8 +145,7 @@ class DeployStackAction(templates.ProcessTemplatesAction):
         stack_is_new = stack is None
 
         # update StackAction, DeployIdentifier and UpdateIdentifier
-        wc = self.get_workflow_client(context)
-        wf_env = wc.environments.get(self.container)
+        swift = self.get_object_client(context)
 
         parameters = dict()
         if not self.skip_deploy_identifier:
@@ -154,15 +153,22 @@ class DeployStackAction(templates.ProcessTemplatesAction):
         parameters['UpdateIdentifier'] = ''
         parameters['StackAction'] = 'CREATE' if stack_is_new else 'UPDATE'
 
-        if 'parameter_defaults' not in wf_env.variables:
-            wf_env.variables['parameter_defaults'] = {}
-        wf_env.variables['parameter_defaults'].update(parameters)
-        env_kwargs = {
-            'name': wf_env.name,
-            'variables': wf_env.variables,
-        }
-        # store params changes back to db before call to process templates
-        wc.environments.update(**env_kwargs)
+        try:
+            env = plan_utils.get_env(swift, self.container)
+        except swiftexceptions.ClientException as err:
+            err_msg = ("Error retrieving environment for plan %s: %s" % (
+                self.container, err))
+            LOG.exception(err_msg)
+            return mistral_workflow_utils.Result(error=err_msg)
+
+        try:
+            plan_utils.update_in_env(swift, env, 'parameter_defaults',
+                                     parameters)
+        except swiftexceptions.ClientException as err:
+            err_msg = ("Error updating environment for plan %s: %s" % (
+                self.container, err))
+            LOG.exception(err_msg)
+            return mistral_workflow_utils.Result(error=err_msg)
 
         # process all plan files and create or update a stack
         processed_data = super(DeployStackAction, self).run(context)
@@ -176,13 +182,12 @@ class DeployStackAction(templates.ProcessTemplatesAction):
         stack_args['timeout_mins'] = self.timeout_mins
 
         if stack_is_new:
-            swift_client = self.get_object_client(context)
             try:
-                swift_client.copy_object(
+                swift.copy_object(
                     "%s-swift-rings" % self.container, "swift-rings.tar.gz",
                     "%s-swift-rings/%s-%d" % (
                         self.container, "swift-rings.tar.gz", time.time()))
-                swift_client.delete_object(
+                swift.delete_object(
                     "%s-swift-rings" % self.container, "swift-rings.tar.gz")
             except swiftexceptions.ClientException:
                 pass
@@ -211,7 +216,7 @@ class OvercloudRcAction(base.TripleOAction):
 
     def run(self, context):
         orchestration_client = self.get_orchestration_client(context)
-        workflow_client = self.get_workflow_client(context)
+        swift = self.get_object_client(context)
 
         try:
             stack = orchestration_client.stacks.get(self.container)
@@ -221,26 +226,27 @@ class OvercloudRcAction(base.TripleOAction):
                 "deployed before calling this action.").format(self.container)
             return mistral_workflow_utils.Result(error=error)
 
-        try:
-            environment = workflow_client.environments.get(self.container)
-        except mistralclient_exc.APIException:
-            error = "The Mistral environment {} could not be found.".format(
-                self.container)
-            return mistral_workflow_utils.Result(error=error)
-
         # We need to check parameter_defaults first for a user provided
         # password. If that doesn't exist, we then should look in the
         # automatically generated passwords.
         # TODO(d0ugal): Abstract this operation somewhere. We shouldn't need to
         # know about the structure of the environment to get a password.
         try:
-            parameter_defaults = environment.variables['parameter_defaults']
-            passwords = environment.variables['passwords']
+            env = plan_utils.get_env(swift, self.container)
+        except swiftexceptions.ClientException as err:
+            err_msg = ("Error retrieving environment for plan %s: %s" % (
+                self.container, err))
+            LOG.error(err_msg)
+            return mistral_workflow_utils.Result(error=err_msg)
+
+        try:
+            parameter_defaults = env['parameter_defaults']
+            passwords = env['passwords']
             admin_pass = parameter_defaults.get('AdminPassword')
             if admin_pass is None:
                 admin_pass = passwords['AdminPassword']
         except KeyError:
-            error = ("Unable to find the AdminPassword in the Mistral "
+            error = ("Unable to find the AdminPassword in the plan "
                      "environment.")
             return mistral_workflow_utils.Result(error=error)
 
