@@ -106,3 +106,15 @@ class RootDeviceDetectionError(Exception):
 
 class PlanOperationError(Exception):
     """Error while performing a deployment plan operation"""
+
+
+class DeriveParamsError(Exception):
+    """Error while performing a derive parameters operation"""
+
+
+class NotFound(Exception):
+    """Resource not found"""
+
+
+class RoleMetadataError(Exception):
+    """Role metadata is invalid"""
