@@ -17,7 +17,6 @@ import zlib
 import mock
 
 from ironicclient.v1 import client as ironicclient
-from mistral import context
 from mistral.utils.openstack import keystone as keystone_utils
 
 from tripleo_common.actions import base
@@ -26,7 +25,6 @@ from tripleo_common.tests import base as tests_base
 from swiftclient.exceptions import ClientException
 
 
-@mock.patch.object(context, 'ctx')
 @mock.patch.object(keystone_utils, 'get_endpoint_for_project')
 class TestActionsBase(tests_base.TestCase):
 
@@ -35,17 +33,18 @@ class TestActionsBase(tests_base.TestCase):
         self.action = base.TripleOAction()
 
     @mock.patch.object(ironicclient, 'Client')
-    def test__get_baremetal_client(self, mock_client, mock_endpoint, mock_cxt):
+    def test__get_baremetal_client(self, mock_client, mock_endpoint):
+        mock_cxt = mock.MagicMock()
         mock_endpoint.return_value = mock.Mock(
             url='http://ironic/v1', region='ironic-region')
-        self.action.get_baremetal_client()
+        self.action.get_baremetal_client(mock_cxt)
         mock_client.assert_called_once_with(
             'http://ironic/v1', max_retries=12, os_ironic_api_version='1.15',
             region_name='ironic-region', retry_interval=5, token=mock.ANY)
         mock_endpoint.assert_called_once_with('ironic')
-        mock_cxt.assert_called_once_with()
+        mock_cxt.assert_not_called()
 
-    def test_cache_key(self, mock_client, mock_endpoint):
+    def test_cache_key(self, mock_endpoint):
         container = "TestContainer"
         key = "testkey"
         cache_key = "__cache_TestContainer_testkey"
@@ -56,7 +55,8 @@ class TestActionsBase(tests_base.TestCase):
         )
 
     @mock.patch("tripleo_common.actions.base.swift_client.Connection")
-    def test_cache_set(self, mock_conn, mock_client, mock_endpoint):
+    def test_cache_set(self, mock_conn, mock_endpoint):
+        mock_ctx = mock.Mock()
         mock_swift = mock.Mock()
         mock_conn.return_value = mock_swift
 
@@ -66,7 +66,7 @@ class TestActionsBase(tests_base.TestCase):
         cache_key = "__cache_TestContainer_testkey"
         compressed_json = zlib.compress("{\"foo\": 1}".encode())
 
-        self.action.cache_set(container, key, {"foo": 1})
+        self.action.cache_set(mock_ctx, container, key, {"foo": 1})
         mock_swift.put_object.assert_called_once_with(
             cache_container,
             cache_key,
@@ -75,7 +75,8 @@ class TestActionsBase(tests_base.TestCase):
         mock_swift.delete_object.assert_not_called()
 
     @mock.patch("tripleo_common.actions.base.swift_client.Connection")
-    def test_cache_set_none(self, mock_conn, mock_client, mock_endpoint):
+    def test_cache_set_none(self, mock_conn, mock_endpoint):
+        mock_ctx = mock.Mock()
         mock_swift = mock.Mock()
         mock_conn.return_value = mock_swift
 
@@ -84,7 +85,7 @@ class TestActionsBase(tests_base.TestCase):
         key = "testkey"
         cache_key = "__cache_TestContainer_testkey"
 
-        self.action.cache_set(container, key, None)
+        self.action.cache_set(mock_ctx, container, key, None)
         mock_swift.put_object.assert_not_called()
         mock_swift.delete_object.called_once_with(
             cache_container,
@@ -92,7 +93,8 @@ class TestActionsBase(tests_base.TestCase):
         )
 
     @mock.patch("tripleo_common.actions.base.swift_client.Connection")
-    def test_cache_get_filled(self, mock_conn, mock_client, mock_endpoint):
+    def test_cache_get_filled(self, mock_conn, mock_endpoint):
+        mock_ctx = mock.Mock()
         mock_swift = mock.Mock()
         mock_conn.return_value = mock_swift
 
@@ -101,11 +103,12 @@ class TestActionsBase(tests_base.TestCase):
         compressed_json = zlib.compress("{\"foo\": 1}".encode())
         # test if cache has something in it
         mock_swift.get_object.return_value = ([], compressed_json)
-        result = self.action.cache_get(container, key)
+        result = self.action.cache_get(mock_ctx, container, key)
         self.assertEqual(result, {"foo": 1})
 
     @mock.patch("tripleo_common.actions.base.swift_client.Connection")
-    def test_cache_empty(self, mock_conn, mock_client, mock_endpoint):
+    def test_cache_empty(self, mock_conn, mock_endpoint):
+        mock_ctx = mock.Mock()
         mock_swift = mock.Mock()
         mock_conn.return_value = mock_swift
 
@@ -117,18 +120,19 @@ class TestActionsBase(tests_base.TestCase):
         mock_swift.get_object.side_effect = ClientException(
             "Foo"
         )
-        result = self.action.cache_get(container, key)
+        result = self.action.cache_get(mock_ctx, container, key)
         self.assertFalse(result)
 
         # delete cache if we have a value
-        self.action.cache_delete(container, key)
+        self.action.cache_delete(mock_ctx, container, key)
         mock_swift.delete_object.assert_called_once_with(
             cache_container,
             cache_key
         )
 
     @mock.patch("tripleo_common.actions.base.swift_client.Connection")
-    def test_cache_delete(self, mock_conn, mock_client, mock_endpoint):
+    def test_cache_delete(self, mock_conn, mock_endpoint):
+        mock_ctx = mock.Mock()
         mock_swift = mock.Mock()
         mock_conn.return_value = mock_swift
 
@@ -139,7 +143,7 @@ class TestActionsBase(tests_base.TestCase):
         mock_swift.delete_object.side_effect = ClientException(
             "Foo"
         )
-        self.action.cache_delete(container, key)
+        self.action.cache_delete(mock_ctx, container, key)
         mock_swift.delete_object.assert_called_once_with(
             cache_container,
             cache_key
