@@ -15,6 +15,7 @@
 import mock
 
 from tripleo_common.actions import package_update
+from tripleo_common import constants
 from tripleo_common.tests import base
 
 
@@ -57,8 +58,7 @@ class UpdateStackActionTest(base.TestCase):
         self.container = 'container'
 
     @mock.patch('tripleo_common.actions.templates.ProcessTemplatesAction.run')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
+    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_orchestration_client')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
@@ -69,10 +69,10 @@ class UpdateStackActionTest(base.TestCase):
                  mock_time,
                  mock_compute_client,
                  mock_orchestration_client,
-                 mock_workflow_client,
-                 mock_templates_run
-                 ):
+                 mock_object_client,
+                 mock_templates_run):
         mock_ctx = mock.MagicMock()
+
         heat = mock.MagicMock()
         heat.stacks.get.return_value = mock.MagicMock(
             stack_name='stack', id='stack_id')
@@ -81,16 +81,17 @@ class UpdateStackActionTest(base.TestCase):
         mock_template_contents.return_value = ({}, {
             'heat_template_version': '2016-04-30'
         })
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.variables = {
-            'temp_environment': 'temp_environment',
-            'template': 'template',
-            'environments': [{u'path': u'environments/test.yaml'}],
-            'parameter_defaults': {'random_data': 'a_value'},
-        }
-        mock_mistral.environments.get.return_value = mock_env
-        mock_workflow_client.return_value = mock_mistral
+        mock_swift = mock.MagicMock()
+        mock_env = """environments:
+- path: environments/test.yaml
+name: container
+parameter_defaults:
+  random_data: a_value
+temp_environment: temp_environment
+template: template
+"""
+        mock_swift.get_object.return_value = ({}, mock_env)
+        mock_object_client.return_value = mock_swift
 
         # freeze time at datetime.datetime(2016, 9, 8, 16, 24, 24)
         mock_time.time.return_value = 1473366264
@@ -106,16 +107,21 @@ class UpdateStackActionTest(base.TestCase):
         action.run(mock_ctx)
 
         # verify parameters are as expected
-        expected_defaults = {
-            'StackAction': 'UPDATE',
-            'DeployIdentifier': 1473366264,
-            'UpdateIdentifier': 1473366264,
-            'random_data': 'a_value',
-        }
-        self.assertEqual(
-            expected_defaults, mock_env.variables['parameter_defaults'])
+        updated_mock_env = """environments:
+- path: environments/test.yaml
+name: container
+parameter_defaults:
+  DeployIdentifier: 1473366264
+  StackAction: UPDATE
+  UpdateIdentifier: 1473366264
+  random_data: a_value
+temp_environment: temp_environment
+template: template
+"""
+        mock_swift.put_object.assert_called_once_with(
+            self.container, constants.PLAN_ENVIRONMENT, updated_mock_env
+        )
 
-        print(heat.mock_calls)
         heat.stacks.update.assert_called_once_with(
             'stack_id',
             StackAction='UPDATE',
