@@ -13,12 +13,15 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 import mock
+import yaml
 
 from swiftclient import exceptions as swiftexceptions
 
 from tripleo_common.actions import parameters
 from tripleo_common import constants
+from tripleo_common import exception
 from tripleo_common.tests import base
+from tripleo_common.utils import passwords as password_utils
 
 _EXISTING_PASSWORDS = {
     'MistralPassword': 'VFJeqBKbatYhQm9jja67hufft',
@@ -64,6 +67,10 @@ _EXISTING_PASSWORDS = {
     'KeystoneCredential1': 'c4MFq82TQLFLKpiiUjrKkp15dafE2ALcD3jbaIu3rfE=',
     'KeystoneFernetKey0': 'O8NSPxr4zXBBAoGIj-5aUmtE7-Jk5a4ptVsEhzJ8Vd8=',
     'KeystoneFernetKey1': 'AueoL37kd6eLjV29AG-Ruxu5szW47osgXx6aPOqtI6I=',
+    'KeystoneFernetKeys': {
+        '/etc/keystone/fernet-keys/0': {'content': 'IAMAVERYSAFEKEY'},
+        '/etc/keystone/fernet-keys/1': {'content': 'IALSOAMAVERYSAFEKEY'}
+    },
     'CephClusterFSID': u'97c16f44-b62c-11e6-aed3-185e0f73fdc5',
     'Ec2ApiPassword': 'FPvz2WiWxrHVWrmSSvv44bqmr',
     'EtcdInitialClusterToken': 'fcVZXehsSc2KdmFFMKDudxTLKa',
@@ -147,11 +154,9 @@ class GetParametersActionTest(base.TestCase):
     @mock.patch('heatclient.common.template_utils.get_template_contents')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_orchestration_client')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
     @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
     def test_run(self, mock_get_object_client,
-                 mock_get_workflow_client, mock_get_orchestration_client,
+                 mock_get_orchestration_client,
                  mock_get_template_contents,
                  mock_process_multiple_environments_and_files,
                  mock_cache_get,
@@ -159,19 +164,17 @@ class GetParametersActionTest(base.TestCase):
 
         mock_ctx = mock.MagicMock()
         swift = mock.MagicMock(url="http://test.com")
-        swift.get_object.side_effect = swiftexceptions.ClientException(
-            'atest2')
-        mock_get_object_client.return_value = swift
-
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.variables = {
+        mock_env = yaml.safe_dump({
             'temp_environment': 'temp_environment',
             'template': 'template',
-            'environments': [{u'path': u'environments/test.yaml'}],
-        }
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+            'environments': [{u'path': u'environments/test.yaml'}]
+        }, default_flow_style=False)
+        swift.get_object.side_effect = (
+            ({}, mock_env),
+            swiftexceptions.ClientException('atest2'),
+            ({}, mock_env)
+        )
+        mock_get_object_client.return_value = swift
 
         mock_get_template_contents.return_value = ({}, {
             'heat_template_version': '2016-04-30'
@@ -201,7 +204,7 @@ class GetParametersActionTest(base.TestCase):
             mock_ctx,
             "overcloud",
             "tripleo.parameters.get",
-            {'heat_resource_tree': {}, 'mistral_environment_parameters': None}
+            {'heat_resource_tree': {}, 'environment_parameters': None}
         )
 
 
@@ -209,31 +212,36 @@ class ResetParametersActionTest(base.TestCase):
 
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'cache_delete')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
-    def test_run(self, mock_get_workflow_client,
-                 mock_cache):
+    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
+    def test_run(self, mock_get_object_client, mock_cache):
 
         mock_ctx = mock.MagicMock()
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.name = constants.DEFAULT_CONTAINER_NAME
-        mock_env.variables = {
+
+        swift = mock.MagicMock(url="http://test.com")
+        mock_env = yaml.safe_dump({
+            'name': constants.DEFAULT_CONTAINER_NAME,
             'template': 'template',
             'environments': [{u'path': u'environments/test.yaml'}],
             'parameter_defaults': {'SomeTestParameter': 42}
-        }
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+        }, default_flow_style=False)
+        swift.get_object.return_value = ({}, mock_env)
+        mock_get_object_client.return_value = swift
+
         # Test
         action = parameters.ResetParametersAction()
+
         action.run(mock_ctx)
-        mock_mistral.environments.update.assert_called_once_with(
-            name=constants.DEFAULT_CONTAINER_NAME,
-            variables={
-                'template': 'template',
-                'environments': [{u'path': u'environments/test.yaml'}],
-            }
+
+        mock_env_reset = yaml.safe_dump({
+            'name': constants.DEFAULT_CONTAINER_NAME,
+            'template': 'template',
+            'environments': [{u'path': u'environments/test.yaml'}]
+        }, default_flow_style=False)
+
+        swift.put_object.assert_called_once_with(
+            constants.DEFAULT_CONTAINER_NAME,
+            constants.PLAN_ENVIRONMENT,
+            mock_env_reset
         )
         mock_cache.assert_called_once_with(
             mock_ctx,
@@ -246,34 +254,38 @@ class UpdateParametersActionTest(base.TestCase):
 
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'cache_delete')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
-    def test_run(self, mock_get_workflow_client, mock_cache):
+    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
+    def test_run(self, mock_get_object_client, mock_cache):
 
         mock_ctx = mock.MagicMock()
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.name = constants.DEFAULT_CONTAINER_NAME
-        mock_env.variables = {
+
+        swift = mock.MagicMock(url="http://test.com")
+        mock_env = yaml.safe_dump({
+            'name': constants.DEFAULT_CONTAINER_NAME,
             'temp_environment': 'temp_environment',
             'template': 'template',
             'environments': [{u'path': u'environments/test.yaml'}],
-        }
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+        }, default_flow_style=False)
+        swift.get_object.return_value = ({}, mock_env)
+        mock_get_object_client.return_value = swift
 
         # Test
         test_parameters = {'SomeTestParameter': 42}
         action = parameters.UpdateParametersAction(test_parameters)
         action.run(mock_ctx)
 
-        mock_mistral.environments.update.assert_called_once_with(
-            name=constants.DEFAULT_CONTAINER_NAME,
-            variables={
-                'temp_environment': 'temp_environment',
-                'template': 'template',
-                'environments': [{u'path': u'environments/test.yaml'}],
-                'parameter_defaults': {'SomeTestParameter': 42}}
+        mock_env_updated = yaml.safe_dump({
+            'name': constants.DEFAULT_CONTAINER_NAME,
+            'temp_environment': 'temp_environment',
+            'parameter_defaults': {'SomeTestParameter': 42},
+            'template': 'template',
+            'environments': [{u'path': u'environments/test.yaml'}]
+        }, default_flow_style=False)
+
+        swift.put_object.assert_called_once_with(
+            constants.DEFAULT_CONTAINER_NAME,
+            constants.PLAN_ENVIRONMENT,
+            mock_env_updated
         )
         mock_cache.assert_called_once_with(
             mock_ctx,
@@ -291,19 +303,18 @@ class UpdateRoleParametersActionTest(base.TestCase):
                 'get_baremetal_client')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_compute_client')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
-    def test_run(self, mock_get_workflow_client,
-                 mock_get_compute_client, mock_get_baremetal_client,
-                 mock_set_count_and_flavor, mock_cache):
+    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
+    def test_run(self, mock_get_object_client, mock_get_compute_client,
+                 mock_get_baremetal_client, mock_set_count_and_flavor,
+                 mock_cache):
 
         mock_ctx = mock.MagicMock()
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.name = 'overcast'
-        mock_env.variables = {}
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+
+        swift = mock.MagicMock(url="http://test.com")
+        mock_env = yaml.safe_dump({'name': 'overcast'},
+                                  default_flow_style=False)
+        swift.get_object.return_value = ({}, mock_env)
+        mock_get_object_client.return_value = swift
 
         params = {'CephStorageCount': 1,
                   'OvercloudCephStorageFlavor': 'ceph-storage'}
@@ -313,8 +324,16 @@ class UpdateRoleParametersActionTest(base.TestCase):
                                                        'overcast')
         action.run(mock_ctx)
 
-        mock_mistral.environments.update.assert_called_once_with(
-            name='overcast', variables={'parameter_defaults': params})
+        mock_env_updated = yaml.safe_dump({
+            'name': 'overcast',
+            'parameter_defaults': params,
+        }, default_flow_style=False)
+
+        swift.put_object.assert_called_once_with(
+            'overcast',
+            constants.PLAN_ENVIRONMENT,
+            mock_env_updated
+        )
         mock_cache.assert_called_once_with(
             mock_ctx,
             "overcast",
@@ -332,23 +351,24 @@ class GeneratePasswordsActionTest(base.TestCase):
                 'get_snmpd_readonly_user_password')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_workflow_client', return_value="TestPassword")
-    def test_run(self, mock_get_workflow_client,
+    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
+    def test_run(self, mock_get_object_client,
+                 mock_get_workflow_client,
                  mock_get_snmpd_readonly_user_password,
                  mock_get_orchestration_client, mock_cache):
 
         mock_get_snmpd_readonly_user_password.return_value = "TestPassword"
-
         mock_ctx = mock.MagicMock()
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.name = constants.DEFAULT_CONTAINER_NAME
-        mock_env.variables = {
+
+        swift = mock.MagicMock(url="http://test.com")
+        mock_env = yaml.safe_dump({
+            'name': 'overcast',
             'temp_environment': 'temp_environment',
             'template': 'template',
             'environments': [{u'path': u'environments/test.yaml'}],
-        }
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+        }, default_flow_style=False)
+        swift.get_object.return_value = ({}, mock_env)
+        mock_get_object_client.return_value = swift
 
         mock_orchestration = mock.MagicMock()
         mock_orchestration.stacks.environment.return_value = {
@@ -374,12 +394,17 @@ class GeneratePasswordsActionTest(base.TestCase):
                 'get_orchestration_client')
     @mock.patch('tripleo_common.utils.passwords.'
                 'create_ssh_keypair')
+    @mock.patch('tripleo_common.utils.passwords.'
+                'create_fernet_keys_repo_structure_and_keys')
     @mock.patch('tripleo_common.utils.passwords.'
                 'get_snmpd_readonly_user_password')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_workflow_client')
-    def test_run_passwords_exist(self, mock_get_workflow_client,
+    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
+    def test_run_passwords_exist(self, mock_get_object_client,
+                                 mock_get_workflow_client,
                                  mock_get_snmpd_readonly_user_password,
+                                 mock_fernet_keys_setup,
                                  mock_create_ssh_keypair,
                                  mock_get_orchestration_client,
                                  mock_cache):
@@ -387,19 +412,21 @@ class GeneratePasswordsActionTest(base.TestCase):
         mock_get_snmpd_readonly_user_password.return_value = "TestPassword"
         mock_create_ssh_keypair.return_value = {'public_key': 'Foo',
                                                 'private_key': 'Bar'}
+        mock_fernet_keys_setup.return_value = {'/tmp/foo': {'content': 'Foo'},
+                                               '/tmp/bar': {'content': 'Bar'}}
 
         mock_ctx = mock.MagicMock()
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.name = constants.DEFAULT_CONTAINER_NAME
-        mock_env.variables = {
+
+        swift = mock.MagicMock(url="http://test.com")
+        mock_env = yaml.safe_dump({
+            'name': constants.DEFAULT_CONTAINER_NAME,
             'temp_environment': 'temp_environment',
             'template': 'template',
             'environments': [{u'path': u'environments/test.yaml'}],
             'passwords': _EXISTING_PASSWORDS.copy()
-        }
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+        }, default_flow_style=False)
+        swift.get_object.return_value = ({}, mock_env)
+        mock_get_object_client.return_value = swift
 
         mock_orchestration = mock.MagicMock()
         mock_orchestration.stacks.environment.return_value = {
@@ -424,12 +451,17 @@ class GeneratePasswordsActionTest(base.TestCase):
                 'get_orchestration_client')
     @mock.patch('tripleo_common.utils.passwords.'
                 'create_ssh_keypair')
+    @mock.patch('tripleo_common.utils.passwords.'
+                'create_fernet_keys_repo_structure_and_keys')
     @mock.patch('tripleo_common.utils.passwords.'
                 'get_snmpd_readonly_user_password')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_workflow_client')
-    def test_passwords_exist_in_heat(self, mock_get_workflow_client,
+    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
+    def test_passwords_exist_in_heat(self, mock_get_object_client,
+                                     mock_get_workflow_client,
                                      mock_get_snmpd_readonly_user_password,
+                                     mock_fernet_keys_setup,
                                      mock_create_ssh_keypair,
                                      mock_get_orchestration_client,
                                      mock_cache):
@@ -437,22 +469,24 @@ class GeneratePasswordsActionTest(base.TestCase):
         mock_get_snmpd_readonly_user_password.return_value = "TestPassword"
         mock_create_ssh_keypair.return_value = {'public_key': 'Foo',
                                                 'private_key': 'Bar'}
+        mock_fernet_keys_setup.return_value = {'/tmp/foo': {'content': 'Foo'},
+                                               '/tmp/bar': {'content': 'Bar'}}
 
         existing_passwords = _EXISTING_PASSWORDS.copy()
         existing_passwords.pop("AdminPassword")
 
         mock_ctx = mock.MagicMock()
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.name = constants.DEFAULT_CONTAINER_NAME
-        mock_env.variables = {
+        swift = mock.MagicMock(url="http://test.com")
+        mock_env = yaml.safe_dump({
+            'name': constants.DEFAULT_CONTAINER_NAME,
             'temp_environment': 'temp_environment',
             'template': 'template',
             'environments': [{u'path': u'environments/test.yaml'}],
             'passwords': existing_passwords.copy()
-        }
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+        }, default_flow_style=False)
+
+        swift.get_object.return_value = ({}, mock_env)
+        mock_get_object_client.return_value = swift
 
         mock_orchestration = mock.MagicMock()
         mock_orchestration.stacks.environment.return_value = {
@@ -479,22 +513,20 @@ class GetPasswordsActionTest(base.TestCase):
 
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_orchestration_client')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
+    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
     def test_password_from_parameter_defaults(self,
-                                              mock_get_workflow_client,
+                                              mock_get_object_client,
                                               mock_get_orchestration_client):
 
         mock_ctx = mock.MagicMock()
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.name = constants.DEFAULT_CONTAINER_NAME
-        mock_env.variables = {
-            "parameter_defaults": _EXISTING_PASSWORDS,
-        }
 
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+        swift = mock.MagicMock(url="http://test.com")
+        mock_env = yaml.safe_dump({
+            "name": constants.DEFAULT_CONTAINER_NAME,
+            "parameter_defaults": _EXISTING_PASSWORDS,
+        }, default_flow_style=False)
+        swift.get_object.return_value = ({}, mock_env)
+        mock_get_object_client.return_value = swift
 
         mock_orchestration = mock.MagicMock()
         mock_get_orchestration_client.return_value = mock_orchestration
@@ -507,24 +539,22 @@ class GetPasswordsActionTest(base.TestCase):
 
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_orchestration_client')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
+    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
     def test_password_from_generated_passwords(self,
-                                               mock_get_workflow_client,
+                                               mock_get_object_client,
                                                mock_get_orchestration_client):
 
         mock_ctx = mock.MagicMock()
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.name = constants.DEFAULT_CONTAINER_NAME
 
-        mock_env.variables = {
+        swift = mock.MagicMock(url="http://test.com")
+        mock_env = yaml.safe_dump({
+            "name": constants.DEFAULT_CONTAINER_NAME,
             "parameter_defaults": {},
             "passwords": _EXISTING_PASSWORDS,
-        }
 
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+        }, default_flow_style=False)
+        swift.get_object.return_value = ({}, mock_env)
+        mock_get_object_client.return_value = swift
 
         mock_orchestration = mock.MagicMock()
         mock_get_orchestration_client.return_value = mock_orchestration
@@ -537,26 +567,25 @@ class GetPasswordsActionTest(base.TestCase):
 
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_orchestration_client')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
+    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
     def test_password_merging_passwords(self,
-                                        mock_get_workflow_client,
+                                        mock_get_object_client,
                                         mock_get_orchestration_client):
 
         parameter_defaults = _EXISTING_PASSWORDS.copy()
         passwords = {"AdminPassword": parameter_defaults.pop("AdminPassword")}
 
         mock_ctx = mock.MagicMock()
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.name = constants.DEFAULT_CONTAINER_NAME
-        mock_env.variables = {
+
+        swift = mock.MagicMock(url="http://test.com")
+        mock_env = yaml.safe_dump({
+            "name": constants.DEFAULT_CONTAINER_NAME,
             "parameter_defaults": parameter_defaults,
             "passwords": passwords
-        }
+        }, default_flow_style=False)
 
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+        swift.get_object.return_value = ({}, mock_env)
+        mock_get_object_client.return_value = swift
 
         mock_orchestration = mock.MagicMock()
         mock_get_orchestration_client.return_value = mock_orchestration
@@ -570,19 +599,13 @@ class GetPasswordsActionTest(base.TestCase):
 
 class GenerateFencingParametersActionTestCase(base.TestCase):
 
-    @mock.patch('tripleo_common.utils.nodes.'
-                'generate_hostmap')
+    @mock.patch('tripleo_common.utils.nodes.generate_hostmap')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_compute_client')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_baremetal_client')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_orchestration_client')
-    def test_no_success(self, mock_get_orchestration,
-                        mock_get_workflow, mock_get_baremetal,
-                        mock_get_compute, mock_generate_hostmap):
+    def test_no_success(self, mock_get_baremetal, mock_get_compute,
+                        mock_generate_hostmap):
         mock_ctx = mock.MagicMock()
         test_hostmap = {
             "00:11:22:33:44:55": {
@@ -678,35 +701,37 @@ class GenerateFencingParametersActionTestCase(base.TestCase):
 
 class GetFlattenedParametersActionTest(base.TestCase):
 
+    @mock.patch('tripleo_common.actions.base.TripleOAction.'
+                'cache_set')
+    @mock.patch('tripleo_common.actions.base.TripleOAction.'
+                'cache_get')
     @mock.patch('heatclient.common.template_utils.'
                 'process_multiple_environments_and_files')
     @mock.patch('heatclient.common.template_utils.get_template_contents')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_orchestration_client')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
     @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
     def test_empty_resource_tree(self, mock_get_object_client,
-                                 mock_get_workflow_client,
                                  mock_get_orchestration_client,
                                  mock_get_template_contents,
-                                 mock_process_multiple_environments_and_files):
+                                 mock_process_multiple_environments_and_files,
+                                 mock_cache_get,
+                                 mock_cache_set):
 
         mock_ctx = mock.MagicMock()
+        mock_cache_get.return_value = None
         swift = mock.MagicMock(url="http://test.com")
-        swift.get_object.side_effect = swiftexceptions.ClientException(
-            'atest2')
-        mock_get_object_client.return_value = swift
-
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.variables = {
+        mock_env = yaml.safe_dump({
             'temp_environment': 'temp_environment',
             'template': 'template',
-            'environments': [{u'path': u'environments/test.yaml'}],
-        }
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+            'environments': [{u'path': u'environments/test.yaml'}]
+        }, default_flow_style=False)
+        swift.get_object.side_effect = (
+            ({}, mock_env),
+            swiftexceptions.ClientException('atest2'),
+            ({}, mock_env)
+        )
+        mock_get_object_client.return_value = swift
 
         mock_get_template_contents.return_value = ({}, {
             'heat_template_version': '2016-04-30'
@@ -720,7 +745,7 @@ class GetFlattenedParametersActionTest(base.TestCase):
 
         expected_value = {
             'heat_resource_tree': {},
-            'mistral_environment_parameters': None,
+            'environment_parameters': None,
         }
 
         # Test
@@ -734,37 +759,39 @@ class GetFlattenedParametersActionTest(base.TestCase):
         )
         self.assertEqual(result, expected_value)
 
+    @mock.patch('tripleo_common.actions.base.TripleOAction.'
+                'cache_set')
+    @mock.patch('tripleo_common.actions.base.TripleOAction.'
+                'cache_get')
     @mock.patch('uuid.uuid4', side_effect=['1', '2'])
     @mock.patch('heatclient.common.template_utils.'
                 'process_multiple_environments_and_files')
     @mock.patch('heatclient.common.template_utils.get_template_contents')
     @mock.patch('tripleo_common.actions.base.TripleOAction.'
                 'get_orchestration_client')
-    @mock.patch('tripleo_common.actions.base.TripleOAction.'
-                'get_workflow_client')
     @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
     def test_valid_resource_tree(self, mock_get_object_client,
-                                 mock_get_workflow_client,
                                  mock_get_orchestration_client,
                                  mock_get_template_contents,
                                  mock_process_multiple_environments_and_files,
-                                 mock_uuid):
+                                 mock_uuid,
+                                 mock_cache_get,
+                                 mock_cache_set):
 
         mock_ctx = mock.MagicMock()
+        mock_cache_get.return_value = None
         swift = mock.MagicMock(url="http://test.com")
-        swift.get_object.side_effect = swiftexceptions.ClientException(
-            'atest2')
-        mock_get_object_client.return_value = swift
-
-        mock_mistral = mock.MagicMock()
-        mock_env = mock.MagicMock()
-        mock_env.variables = {
+        mock_env = yaml.safe_dump({
             'temp_environment': 'temp_environment',
             'template': 'template',
-            'environments': [{u'path': u'environments/test.yaml'}],
-        }
-        mock_mistral.environments.get.return_value = mock_env
-        mock_get_workflow_client.return_value = mock_mistral
+            'environments': [{u'path': u'environments/test.yaml'}]
+        }, default_flow_style=False)
+        swift.get_object.side_effect = (
+            ({}, mock_env),
+            swiftexceptions.ClientException('atest2'),
+            ({}, mock_env)
+        )
+        mock_get_object_client.return_value = swift
 
         mock_get_template_contents.return_value = ({}, {
             'heat_template_version': '2016-04-30'
@@ -816,10 +843,128 @@ class GetFlattenedParametersActionTest(base.TestCase):
                     }
                 },
             },
-            'mistral_environment_parameters': None,
+            'environment_parameters': None,
         }
 
         # Test
         action = parameters.GetFlattenedParametersAction()
         result = action.run(mock_ctx)
         self.assertEqual(result, expected_value)
+
+
+class GetProfileOfFlavorActionTest(base.TestCase):
+
+    @mock.patch('tripleo_common.utils.parameters.get_profile_of_flavor')
+    @mock.patch('tripleo_common.actions.base.TripleOAction.'
+                'get_compute_client')
+    @mock.patch('mistral.context.ctx')
+    def test_profile_found(self, mock_ctx, mock_get_compute_client,
+                           mock_get_profile_of_flavor):
+        mock_ctx = mock.MagicMock()
+        mock_get_profile_of_flavor.return_value = 'compute'
+        action = parameters.GetProfileOfFlavorAction('oooq_compute')
+        result = action.run(mock_ctx)
+        expected_result = "compute"
+        self.assertEqual(result, expected_result)
+
+    @mock.patch('tripleo_common.utils.parameters.get_profile_of_flavor')
+    @mock.patch('tripleo_common.actions.base.TripleOAction.'
+                'get_compute_client')
+    @mock.patch('mistral.context.ctx')
+    def test_profile_not_found(self, mock_ctx, mock_get_compute_client,
+                               mock_get_profile_of_flavor):
+        mock_ctx = mock.MagicMock()
+        profile = (exception.DeriveParamsError, )
+        mock_get_profile_of_flavor.side_effect = profile
+        action = parameters.GetProfileOfFlavorAction('no_profile')
+        result = action.run(mock_ctx)
+        self.assertTrue(result.is_error())
+        mock_get_profile_of_flavor.assert_called_once()
+
+
+class RotateFernetKeysActionTest(base.TestCase):
+
+    def test_get_next_index(self):
+        action = parameters.RotateFernetKeysAction()
+        keys_map = {
+            password_utils.KEYSTONE_FERNET_REPO + '0': {
+                'content': 'Some key'},
+            password_utils.KEYSTONE_FERNET_REPO + '1': {
+                'content': 'Some other key'},
+        }
+        next_index = action.get_next_index(keys_map)
+        self.assertEqual(next_index, 2)
+
+    @mock.patch('tripleo_common.utils.passwords.'
+                'create_keystone_credential')
+    def test_rotate_keys(self, mock_keystone_creds):
+        action = parameters.RotateFernetKeysAction()
+        mock_keystone_creds.return_value = 'Some new key'
+
+        staged_key_index = password_utils.KEYSTONE_FERNET_REPO + '0'
+        new_primary_key_index = password_utils.KEYSTONE_FERNET_REPO + '2'
+        keys_map = {
+            password_utils.KEYSTONE_FERNET_REPO + '0': {
+                'content': 'Some key'},
+            password_utils.KEYSTONE_FERNET_REPO + '1': {
+                'content': 'Some other key'},
+        }
+        new_keys_map = action.rotate_keys(keys_map, 2)
+
+        # Staged key should be the new key
+        self.assertEqual('Some new key',
+                         new_keys_map[staged_key_index]['content'])
+        # primary key should be the previous staged key
+        self.assertEqual('Some key',
+                         new_keys_map[new_primary_key_index]['content'])
+
+    def test_purge_excess_keys_should_purge(self):
+        action = parameters.RotateFernetKeysAction()
+        keys_map = {
+            password_utils.KEYSTONE_FERNET_REPO + '0': {
+                'content': 'key0'},
+            password_utils.KEYSTONE_FERNET_REPO + '1': {
+                'content': 'key1'},
+            password_utils.KEYSTONE_FERNET_REPO + '2': {
+                'content': 'key2'},
+            password_utils.KEYSTONE_FERNET_REPO + '3': {
+                'content': 'key3'},
+            password_utils.KEYSTONE_FERNET_REPO + '4': {
+                'content': 'key4'},
+        }
+        max_keys = 3
+        keys_map = action.purge_excess_keys(max_keys, keys_map)
+        self.assertEqual(max_keys, len(keys_map))
+        # It should keep index 0, 3 and 4
+        self.assertIn(password_utils.KEYSTONE_FERNET_REPO + '0', keys_map)
+        self.assertIn(password_utils.KEYSTONE_FERNET_REPO + '3', keys_map)
+        self.assertIn(password_utils.KEYSTONE_FERNET_REPO + '4', keys_map)
+        # It sould have removed index 1 and 2
+        self.assertNotIn(password_utils.KEYSTONE_FERNET_REPO + '1', keys_map)
+        self.assertNotIn(password_utils.KEYSTONE_FERNET_REPO + '2', keys_map)
+
+    def test_purge_excess_keys_should_not_purge_if_equal_to_max(self):
+        action = parameters.RotateFernetKeysAction()
+        keys_map = {
+            password_utils.KEYSTONE_FERNET_REPO + '0': {
+                'content': 'key0'},
+            password_utils.KEYSTONE_FERNET_REPO + '1': {
+                'content': 'key1'},
+            password_utils.KEYSTONE_FERNET_REPO + '2': {
+                'content': 'key2'},
+        }
+        max_keys = 3
+        keys_map = action.purge_excess_keys(max_keys, keys_map)
+        self.assertEqual(max_keys, len(keys_map))
+
+    def test_purge_excess_keys_should_not_purge_if_less_than_max(self):
+        action = parameters.RotateFernetKeysAction()
+        keys_map = {
+            password_utils.KEYSTONE_FERNET_REPO + '0': {
+                'content': 'key0'},
+            password_utils.KEYSTONE_FERNET_REPO + '1': {
+                'content': 'key1'},
+        }
+        max_keys = 3
+        keys_map = action.purge_excess_keys(max_keys, keys_map)
+        self.assertEqual(2, len(keys_map))
