@@ -27,10 +27,8 @@ from tripleo_common.tests.image import fakes
 
 
 filedata = six.u(
-    """uploads:
+    """container_images:
     - imagename: tripleoupstream/heat-docker-agents-centos:latest
-      uploader: docker
-      pull_source: docker.io
       push_destination: localhost:8787
     - imagename: tripleoupstream/centos-binary-nova-compute:liberty
       uploader: docker
@@ -39,6 +37,22 @@ filedata = six.u(
     - imagename: tripleoupstream/centos-binary-nova-libvirt:liberty
       uploader: docker
       pull_source: docker.io
+    - imagename: tripleoupstream/image-with-missing-tag
+      push_destination: localhost:8787
+""")
+
+legacy_filedata = six.u(
+    """uploads:
+    - imagename: tripleoupstream/heat-docker-agents-centos:latest
+      push_destination: localhost:8787
+    - imagename: tripleoupstream/centos-binary-nova-compute:liberty
+      uploader: docker
+      pull_source: docker.io
+      push_destination: localhost:8787
+    - imagename: tripleoupstream/centos-binary-nova-libvirt:liberty
+      uploader: docker
+      pull_source: docker.io
+    - imagename: tripleoupstream/image-with-missing-tag
       push_destination: localhost:8787
 """)
 
@@ -53,8 +67,9 @@ class TestImageUploadManager(base.TestCase):
     @mock.patch('tripleo_common.image.base.open',
                 mock.mock_open(read_data=filedata), create=True)
     @mock.patch('os.path.isfile', return_value=True)
+    @mock.patch('fcntl.ioctl', side_effect=Exception)
     @mock.patch('tripleo_common.image.image_uploader.Client')
-    def test_file_parsing(self, mockpath, mockdocker):
+    def test_file_parsing(self, mockpath, mockioctl, mockdocker):
         print(filedata)
         manager = ImageUploadManager(self.filelist, debug=True)
         parsed_data = manager.upload()
@@ -65,7 +80,24 @@ class TestImageUploadManager(base.TestCase):
                                       key=operator.itemgetter('imagename'))
         sorted_parsed_data = sorted(parsed_data,
                                     key=operator.itemgetter('imagename'))
-        self.assertEqual(sorted_parsed_data, sorted_expected_data)
+        self.assertEqual(sorted_expected_data, sorted_parsed_data)
+
+    @mock.patch('tripleo_common.image.base.open',
+                mock.mock_open(read_data=legacy_filedata), create=True)
+    @mock.patch('os.path.isfile', return_value=True)
+    @mock.patch('fcntl.ioctl', side_effect=Exception)
+    @mock.patch('tripleo_common.image.image_uploader.Client')
+    def test_legacy_file_parsing(self, mockpath, mockioctl, mockdocker):
+        manager = ImageUploadManager(self.filelist, debug=True)
+        parsed_data = manager.upload()
+        mockpath(self.filelist[0])
+
+        expected_data = fakes.create_parsed_upload_images()
+        sorted_expected_data = sorted(expected_data,
+                                      key=operator.itemgetter('imagename'))
+        sorted_parsed_data = sorted(parsed_data,
+                                    key=operator.itemgetter('imagename'))
+        self.assertEqual(sorted_expected_data, sorted_parsed_data)
 
 
 class TestImageUploader(base.TestCase):
@@ -117,3 +149,27 @@ class TestDockerImageUploader(base.TestCase):
         self.dockermock.return_value.push(
             push_destination + '/' + image,
             tag=tag, stream=True, insecure_registry=True)
+
+    def test_upload_image_missing_tag(self):
+        image = 'tripleoupstream/heat-docker-agents-centos'
+        expected_tag = 'latest'
+        pull_source = 'docker.io'
+        push_destination = 'localhost:8787'
+
+        self.uploader.upload_image(image,
+                                   pull_source,
+                                   push_destination)
+
+        self.dockermock.assert_called_once_with(
+            base_url='unix://var/run/docker.sock')
+
+        self.dockermock.return_value.pull.assert_called_once_with(
+            pull_source + '/' + image,
+            tag=expected_tag, stream=True, insecure_registry=True)
+        self.dockermock.return_value.tag.assert_called_once_with(
+            image=pull_source + '/' + image + ':' + expected_tag,
+            repository=push_destination + '/' + image,
+            tag=expected_tag, force=True)
+        self.dockermock.return_value.push(
+            push_destination + '/' + image,
+            tag=expected_tag, stream=True, insecure_registry=True)
